@@ -9,6 +9,7 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE UndecidableInstances #-}
+{-# LANGUAGE Trustworthy #-}
 -- |  This module provides the facilities needed for a decoupled logging system.
 --
 -- The 'MonadLogger' class is implemented by monads that give access to a
@@ -69,7 +70,7 @@ module Control.Monad.Logger
 
     -- * utilities for defining your own loggers
     , defaultLogStr
-    , Loc
+    , Loc (..)
     ) where
 
 #if WITH_TEMPLATE_HASKELL
@@ -132,13 +133,15 @@ import Control.Monad.Reader.Class ( MonadReader (..) )
 import Control.Monad.State.Class  ( MonadState (..) )
 import Control.Monad.Writer.Class ( MonadWriter (..) )
 
-import Blaze.ByteString.Builder (toByteString)
-
 import Prelude hiding (catch)
 
-#if !MIN_VERSION_fast_logger(2, 1, 0) && MIN_VERSION_bytestring(0, 10, 2)
+#if MIN_VERSION_fast_logger(2, 1, 0)
+-- Using System.Log.FastLogger
+#elif MIN_VERSION_bytestring(0, 10, 2)
 import qualified Data.ByteString.Lazy as L
 import Data.ByteString.Builder (toLazyByteString)
+#else
+import Blaze.ByteString.Builder (toByteString)
 #endif
 
 #if MIN_VERSION_conduit_extra(1,1,0)
@@ -174,6 +177,9 @@ type CharPos = (Int, Int)
 -- | A @Monad@ which has the ability to log messages in some manner.
 class Monad m => MonadLogger m where
     monadLoggerLog :: ToLogStr msg => Loc -> LogSource -> LogLevel -> msg -> m ()
+    default monadLoggerLog :: (Trans.MonadTrans t, MonadLogger (t m), ToLogStr msg)
+                           => Loc -> LogSource -> LogLevel -> msg -> t m ()
+    monadLoggerLog loc src lvl msg = Trans.lift $ monadLoggerLog loc src lvl msg
 
 -- | An extension of @MonadLogger@ for the common case where the logging action
 -- is a simple @IO@ action. The advantage of using this typeclass is that the
@@ -515,9 +521,12 @@ defaultLogStr loc src level msg =
         else "#" `mappend` toLogStr src) `mappend`
     "] " `mappend`
     msg `mappend`
-    " @(" `mappend`
-    toLogStr (S8.pack fileLocStr) `mappend`
-    ")\n"
+    (if isDefaultLoc loc
+        then "\n"
+        else
+            " @(" `mappend`
+            toLogStr (S8.pack fileLocStr) `mappend`
+            ")\n")
 #else
     S8.concat
         [ S8.pack "["
@@ -576,7 +585,7 @@ runStdoutLoggingT = (`runLoggingT` defaultOutput stdout)
 --
 -- Since 0.3.2
 withChannelLogger :: (MonadBaseControl IO m, MonadIO m)
-                  => Int         -- ^ Number of mesasges to keep
+                  => Int         -- ^ Number of messages to keep
                   -> LoggingT m a
                   -> LoggingT m a
 withChannelLogger size action = LoggingT $ \logger -> do
@@ -647,6 +656,10 @@ instance MonadWriter w m => MonadWriter w (NoLoggingT m) where
 defaultLoc :: Loc
 defaultLoc = Loc "<unknown>" "<unknown>" "<unknown>" (0,0) (0,0)
 
+isDefaultLoc :: Loc -> Bool
+isDefaultLoc (Loc "<unknown>" "<unknown>" "<unknown>" (0,0) (0,0)) = True
+isDefaultLoc _ = False
+
 logWithoutLoc :: (MonadLogger m, ToLogStr msg) => LogSource -> LogLevel -> msg -> m ()
 logWithoutLoc = monadLoggerLog defaultLoc
 
